#   -*- coding: utf-8 -*-

##############################################################################
# copyrights and license
#
# Copyright (c) 2025 David Harris Kiesel
#
# Licensed under the MIT License. See LICENSE in the project root for license
# information.
##############################################################################

from copy import copy
import csv
from datetime import (
    date,
    datetime,
)
from decimal import Decimal
from typing import Iterable

# https://pypi.org/project/XlsxWriter/
# https://xlsxwriter.readthedocs.io/
# https://github.com/jmcnamara/XlsxWriter
import xlsxwriter


def parse_bool(
    s: str
) -> bool:
    'Parse a Boolean value.'

    ls = s.lower()

    if ls in (
        'true',
        't',
        'yes',
        'y',
        'on',
        '1',
    ):
        return True
    elif ls in (
        'false',
        'f',
        'no',
        'n',
        'off',
        '0'
    ):
        return False
    else:
        raise \
            ValueError(
                'Invalid value.'
            )


def get_first_and_last_columns(
    column_setting: dict,
    next_col: int
) -> tuple[int, int]:
    'Get first and last columns.'

    columns = column_setting.get('columns')

    if columns is not None:
        if columns.find(':') < 0:
            columns = \
                '{0}:{0}'.format(
                    columns
                )

        columns_components = \
            columns.split(':')

        (row, first_col) = \
            xlsxwriter.utility.xl_cell_to_rowcol(
                columns_components[0] + '1'
            )

        (row, last_col) = \
            xlsxwriter.utility.xl_cell_to_rowcol(
                columns_components[1] + '1'
            )
    else:
        first_col = next_col

        last_col = (
            next_col
            + column_setting.get(
                'span',
                1
            )
            - 1
        )

    return (first_col, last_col)


def get_first_and_last_rows(
    row_setting: dict,
    next_row: int
) -> tuple[int, int]:
    'Get first and last rows.'

    rows = row_setting.get('rows')

    if rows is not None:
        if rows.find(':') < 0:
            first_row = int(rows)

            last_row = int(rows)
        else:
            (first_row, last_row) = \
                [int(x) for x in rows.split(':')]
    else:
        first_row = next_row

        last_row = (
            next_row
            + row_setting.get(
                'span',
                1
            )
            - 1
        )

    return (first_row, last_row)


def get_workbook_format_setters(
    workbook: xlsxwriter.Workbook
) -> dict:
    'Get Workbook format setters.'

    return \
        {
            'font_name': workbook.formats[0].set_font_name,
            'font_size': workbook.formats[0].set_font_size,
            'font_color': workbook.formats[0].set_font_color,
            'bold': workbook.formats[0].set_bold,
            'italic': workbook.formats[0].set_italic,
            'underline': workbook.formats[0].set_underline,
            'font_strikeout': workbook.formats[0].set_font_strikeout,
            'font_script': workbook.formats[0].set_font_script,
            'num_format': workbook.formats[0].set_num_format,
            'locked': workbook.formats[0].set_locked,
            'hidden': workbook.formats[0].set_hidden,
            'align': workbook.formats[0].set_align,
            'valign': workbook.formats[0].set_align,
            'rotation': workbook.formats[0].set_rotation,
            'text_wrap': workbook.formats[0].set_text_wrap,
            'reading_order': workbook.formats[0].set_reading_order,
            'text_justlast': workbook.formats[0].set_text_justlast,
            'center_across': workbook.formats[0].set_center_across,
            'indent': workbook.formats[0].set_indent,
            'shrink': workbook.formats[0].set_shrink,
            'pattern': workbook.formats[0].set_pattern,
            'bg_color': workbook.formats[0].set_bg_color,
            'fg_color': workbook.formats[0].set_fg_color,
            'border': workbook.formats[0].set_border,
            'bottom': workbook.formats[0].set_bottom,
            'top': workbook.formats[0].set_top,
            'left': workbook.formats[0].set_left,
            'right': workbook.formats[0].set_right,
            'border_color': workbook.formats[0].set_border_color,
            'bottom_color': workbook.formats[0].set_bottom_color,
            'top_color': workbook.formats[0].set_top_color,
            'left_color': workbook.formats[0].set_left_color,
            'right_color': workbook.formats[0].set_right_color,
        }


def apply_workbook_format(
    workbook: xlsxwriter.Workbook,
    workbook_format: dict
) -> None:
    'Apply Workbook format.'

    format_setters = \
        get_workbook_format_setters(
            workbook
        )

    for setting_name, setting_value in workbook_format.items():
        format_setter = format_setters.get(setting_name)

        if format_setter is not None:
            format_setter(setting_value)


def transform(
    csv_iterable: Iterable[str],
    workbook_path: str,
    workbook_settings: dict = None
) -> None:
    'Read a CSV iterable and write an XLSX file.'

    if workbook_settings is None:
        workbook_settings = {}

    workbook_options = \
        (
            {
                'constant_memory': True
            }
            | workbook_settings.get(
                'workbook_options',
                {}
            )
        )

    worksheet_settings = \
        workbook_settings.get(
            'worksheet_settings',
            {}
        )

    workbook_format = \
        workbook_settings.get(
            'workbook_format',
            {}
        )

    cell_format_settings = \
        workbook_settings.get(
            'cell_format_settings',
            {}
        )

    column_settings = \
        workbook_settings.get(
            'column_settings',
            []
        )

    row_settings = \
        workbook_settings.get(
            'row_settings',
            []
        )

    with xlsxwriter.Workbook(
        filename=workbook_path,
        options=workbook_options
    ) as workbook:
        apply_workbook_format(
            workbook,
            workbook_format
        )

        cell_formats = {}

        for cell_format_name, cell_format in cell_format_settings.items():
            cell_formats[cell_format_name] = \
                workbook.add_format(
                    workbook_format
                    | cell_format
                )

        worksheet = \
            workbook.add_worksheet()

        default_width = 10

        column_data_types = {}

        next_col = 0

        for column_setting in column_settings:
            (first_col, last_col) = \
                get_first_and_last_columns(
                    column_setting,
                    next_col
                )

            worksheet.set_column(
                first_col,
                last_col,
                width=column_setting.get(
                    'width',
                    default_width
                ),
                cell_format=cell_formats.get(
                    column_setting.get(
                        'cell_format'
                    ),
                    workbook.formats[0]
                ),
                options=column_setting.get('options')
            )

            for col in range(
                first_col,
                last_col + 1
            ):
                column_data_types[col] = \
                    column_setting.get('data_type')

            next_col = last_col + 1

        row_settings_dict = {}

        next_row = 0

        for row_setting in row_settings:
            (first_row, last_row) = \
                get_first_and_last_rows(
                    row_setting,
                    next_row
                )

            for row in range(first_row, last_row + 1):
                worksheet.set_row(
                    row,
                    cell_format=cell_formats.get(
                        row_setting.get(
                            'cell_format'
                        ),
                        workbook.formats[0]
                    )
                )

                row_settings_dict[row] = row_setting

            next_row = last_row + 1

        csv_reader = \
            csv.reader(
                csv_iterable
            )

        max_column_offset = -1

        for row_offset, row in enumerate(csv_reader):
            row_setting = row_settings_dict.get(row_offset)

            for column_offset, cell_value in enumerate(row):
                if row_setting is not None:
                    data_type = \
                        row_setting.get('data_type')
                else:
                    data_type = \
                        column_data_types.get(column_offset)

                if data_type == 'decimal':
                    try:
                        converted_cell_value = \
                            Decimal(cell_value)

                        worksheet.write(
                            row_offset,
                            column_offset,
                            converted_cell_value
                        )
                    except Exception:
                        worksheet.write_string(
                            row_offset,
                            column_offset,
                            cell_value
                        )
                elif data_type == 'bool':
                    try:
                        converted_cell_value = \
                            parse_bool(cell_value)

                        worksheet.write(
                            row_offset,
                            column_offset,
                            converted_cell_value
                        )
                    except Exception:
                        worksheet.write_string(
                            row_offset,
                            column_offset,
                            cell_value
                        )
                elif data_type == 'date':
                    try:
                        converted_cell_value = \
                            date.fromisoformat(cell_value)

                        worksheet.write(
                            row_offset,
                            column_offset,
                            converted_cell_value
                        )
                    except Exception:
                        worksheet.write_string(
                            row_offset,
                            column_offset,
                            cell_value
                        )
                elif data_type == 'datetime':
                    try:
                        converted_cell_value = \
                            datetime.fromisoformat(cell_value)

                        worksheet.write(
                            row_offset,
                            column_offset,
                            converted_cell_value
                        )
                    except Exception:
                        worksheet.write_string(
                            row_offset,
                            column_offset,
                            cell_value
                        )
                elif data_type == 'formula':
                    worksheet.write_formula(
                        row_offset,
                        column_offset,
                        cell_value
                    )
                else:
                    worksheet.write_string(
                        row_offset,
                        column_offset,
                        cell_value
                    )

                if column_offset > max_column_offset:
                    max_column_offset = column_offset

        autofilter = \
            copy(
                worksheet_settings.get('autofilter')
            )

        if autofilter is not None:
            if isinstance(autofilter, list):
                if (
                    autofilter[2] > row_offset
                    or autofilter[2] < 0
                ):
                    autofilter[2] = row_offset

                if (
                    autofilter[3] > max_column_offset
                    or autofilter[3] < 0
                ):
                    autofilter[3] = max_column_offset

                worksheet.autofilter(
                    *autofilter
                )
            else:
                worksheet.autofilter(
                    autofilter
                )

        freeze_panes = \
            worksheet_settings.get('freeze_panes')

        if freeze_panes is not None:
            if isinstance(freeze_panes, list):
                worksheet.freeze_panes(
                    *freeze_panes
                )
            else:
                worksheet.freeze_panes(
                    freeze_panes
                )



#   -*- coding: utf-8 -*-

##############################################################################
# copyrights and license
#
# Copyright (c) 2025 David Harris Kiesel
#
# Licensed under the MIT License. See LICENSE in the project root for license
# information.
##############################################################################

import argparse
import json
import logging
import os.path
import sys

from dhk.csv2xlsx import csv2xlsx


sample_settings = \
    '''{
    "workbook_options": {
    },
    "workbook_format": {
        "font_size": 12,
        "valign": "top"
    },
    "worksheet_settings": {
        "freeze_panes": [
            1,
            0
        ],
        "autofilter": [
            0,
            0,
            -1,
            -1
        ]
    },
    "cell_format_settings": {
        "header": {
            "valign": "bottom",
            "bold": true,
            "text_wrap": true
        },
        "date": {
            "num_format": "yyyy-mm-dd"
        },
        "datetime": {
            "num_format": "yyyy-mm-dd HH:MM:SS.000"
        },
        "num0": {
            "num_format": "0"
        },
        "numc0": {
            "num_format": "#,##0"
        },
        "num2": {
            "num_format": "0.00"
        },
        "numc2": {
            "num_format": "#,##0.00"
        },
        "dollar0": {
            "num_format": "$#,##0"
        },
        "dollar2": {
            "num_format": "$#,##0.00"
        },
        "textwrap": {
            "text_wrap": true
        }
    },
    "column_settings": [
        {
            "span": 2,
            "width": 40,
            "cell_format": "textwrap"
        },
        {
            "width": 40
        },
        {
            "data_type": "bool",
            "width": 10
        },
        {
            "data_type": "date",
            "width": 20,
            "cell_format": "date"
        },
        {
            "data_type": "datetime",
            "width": 40,
            "cell_format": "datetime"
        },
        {
            "data_type": "decimal",
            "width": 20,
            "cell_format": "num0",
            "options": {
                "hidden": true
            }
        },
        {
            "data_type": "decimal",
            "width": 20,
            "cell_format": "dollar2"
        }
    ],
    "row_settings": [
        {
            "data_type": "string",
            "cell_format": "header"
        }
    ]
}'''


def get_parser() -> argparse.ArgumentParser:
    'Get parser.'

    parser = \
        argparse.ArgumentParser(
            prog='csv2xlsx',
            formatter_class=argparse.RawDescriptionHelpFormatter,
            description='''Read a CSV file and write an XLSX file.

The program exposes much of the formatting functionality of the XlsxWriter
packages Workbook class through the optional SETTINGS_FILE.  For details about
the Workbook class, see https://xlsxwriter.readthedocs.io/workbook.html.  For
details about the Worksheet class, see
https://xlsxwriter.readthedocs.io/worksheet.html.  For details about the Format
class, see https://xlsxwriter.readthedocs.io/format.html.
''',
            add_help=True,
            epilog="""
examples:

    %(prog)s \\
        --generate-settings-file

    %(prog)s \\
        CSV_FILE

    %(prog)s \\
        --settings-file SETTINGS_FILE \\
        CSV_FILE

    %(prog)s \\
        --settings-file SETTINGS_FILE \\
        --output OUTPUT \\
        CSV_FILE

execution details:

The Workbook class instance attribute `options` is set to `{'constant_memory':
True}` merged with the SETTINGS_FILE `workbook_options` dictionary.

The Workbook class instance attribute `formats[0]` stores a default Format
object.  The SETTINGS_FILE `workbook_format` dictionary can be used to set the
attributes of this default Format object.  The available dictionary keywords
correspond to the list of properties in the table at
https://xlsxwriter.readthedocs.io/format.html#format-methods-and-format-properties.

The Workbook class instance method `add_format()` is used to add Format class
instances to the Workbook class instance.  The program iterates over each of
the SETTINGS_FILE `cell_format_settings` dictionary key-value pairs.  The
SETTINGS_FILE `workbook_format` dictionary is merged with the value of the
`cell_format_settings` key-value pair, and this merged value is the argument to
`add_format()`.  The key-value pair formed by the key from the
`cell_format_settings` key-value pair and the `add_format()` return value is
stored in a `cell_formats` dictionary for later use when setting Worksheet
class instance columns and rows.

The Worksheet class instance method `set_column()` is used to set the width,
cell format, and options for specified columns.  The program iterates over each
of the SETTINGS_FILE `column_settings` list elements.  For each element, the
`first_col` and `last_col` arguments to `set_column()` can be specified in
absolute terms via a `columns` key-value pair with the string value specified
by ordinal number (`'1'`, `'1:2'`, etc.) or by `A1` style notation (`'A'`,
`'A:B'`).  Alternatively, if the `columns` key-value pair is not present, then
`first_col` is simply the next column, and `last_col` is either the same as
`first_col` or is set based on a `span` key-value pair.  The `width` argument
to `set_column()` is set to the value of the `width` key-value pair, defaulting
to 10 if not specified.  The `cell_format` argument to `set_column()` is set
based on the value of the `cell_format` key-value pair.  This value must
correspond to the key of one of the SETTINGS_FILE `cell_format_settings`
key-value pairs.  If a `cell_format` key-value pair is not provided, then the
argument defaults to the Workbook class instance attribute `formats[0]`.  The
`options` argument to `set_column()` is set to the value of the `options`
key-pair.  The value of the `data_type` key-pair is stored for each column for
later use when parsing CSV data.

The Worksheet class instance method `set_row()` is used to set the cell format
for specified rows.  The program iterates over each of the SETTINGS_FILE
`row_settings` list elements.  For each element, `first_row` and `last_row` can
be specified in absolute terms via a `rows` key-value pair with the string
value specified by ordinal number (`'1'`, `'1:2'`, etc.).  Alternatively, if
the `rows` key-value pair is not present, then `first_row` is simply the next
column, and `last_row` is either the same as `first_row` or is set based on a
`span` key-value pair.  For each `row` in the sequence determined by
`first_row` and `last_row`, `set_row()` is called with `row` as the first
positional argument and the argument `cell_format` set based on the value of
the `cell_format` key-value pair.  This value must correspond to the key of one
of the SETTINGS_FILE `cell_format_settings` key-value pairs.  If a
`cell_format` key-value pair is not provided, then the argument defaults to
the Workbook class instance attribute `formats[0]`.  The value of the
`data_type` key-pair is stored for each row for later use when parsing CSV
data.

Each row of the CSV file is read.  The value of each column of the row is
handled.  If there was a `data_type` specified in the SETTINGS_FILE for the
row, then that is used.  Otherwise, an attempt is made to get a `data_type`
that was specified in the SETTINGS_FILE for the column.  Valid values for
`data_type` are: `decimal`, `bool`, `date`, `datetime`, `formula`, and
`string`.  If `data_type` was not specified or an exception occurs when
attempting to parse the CSV value in terms of the given `data_type`, then the
CSV value is written as a `string`.

The Worksheet class instance method `autofilter` is called with arguments based
on the `autofilter` object in the SETTINGS_FILE `workbook_settings` dictionary.
Both row-column notation (`(first_row, first_col, last_row, last_col)`) and
`A1` style notation (`'A1:D11'`) are supported.  If row-column notation is
used, then a negative number is replaced with the maximum row or column
available in the data.

The Worksheet class instance method `freeze_panes` is called with arguments
based on the `freeze_panes` object in the SETTINGS_FILE `workbook_settings`
dictionary.  Both row-column notation (`(row, col[, top_row, left_col])`) and
`A1` style notation (`'A2'`) are supported.
"""
        )

    parser.add_argument(
        '--force',
        '-f',
        action='store_true',
        help='force; suppress prompts'
    )

    parser.add_argument(
        '--output',
        '-o',
        dest='output_file',
        default=None,
        help='output file; default: CSV_FILE - .csv + .xlsx'
    )

    parser.add_argument(
        '--settings-file',
        '-s',
        metavar='SETTINGS_FILE',
        default=None,
        help='settings file'
    )

    parser.add_argument(
        '--verbose',
        '-v',
        action='count',
        default=0,
        help='verbose'
    )

    parser.add_argument(
        'csv_fd',
        metavar='CSV_FILE',
        type=argparse.FileType('r'),
        nargs='?',
        help='CSV file'
    )

    mutually_exclusive_group = \
        parser.add_mutually_exclusive_group()

    mutually_exclusive_group.add_argument(
        '--generate-settings-file',
        '-g',
        action='store_true',
        help='generate settings file; file defaults to sample.settings.json'
    )

    mutually_exclusive_group.add_argument(
        '--transform-csv',
        '-t',
        action='store_true',
        help='transform CSV to XLSX; default True'
    )

    return parser


def configure_logging(
    level: int
) -> None:
    'Configure logging.'

    logging.basicConfig(
        level=level
    )


def main(
    args: argparse.Namespace
) -> None:
    '``main`` entry point.'

    logging_levels = \
        (
            logging.WARNING,
            logging.INFO,
            logging.DEBUG,
        )

    configure_logging(
        logging_levels[
            min(
                len(logging_levels) - 1,
                args.verbose
            )
        ]
    )

    if args.generate_settings_file:
        for incompatible_option in (
            args.output_file,
        ):
            if incompatible_option is not None:
                print(
                    'Incompatible option.'
                )

                sys.exit()

        if args.settings_file is None:
            args.settings_file = 'sample.settings.json'

        if not args.force:
            while os.path.exists(args.settings_file):
                response = \
                    input(
                        "Settings file '{0}' exists.  Continue? (y/n) ".format(
                            args.settings_file
                        )
                    )

                if response == 'y':
                    break
                elif response == 'n':
                    sys.exit()

        with open(
            args.settings_file,
            'w',
            encoding='utf-8'
        ) as settings_fd:
            print(
                sample_settings,
                file=settings_fd
            )
    else:
        if args.settings_file is not None:
            with open(
                args.settings_file,
                'r'
            ) as settings_fd:
                workbook_settings = \
                    json.load(settings_fd)
        else:
            workbook_settings = {}

        workbook_path = args.output_file

        if workbook_path is None:
            root, ext = \
                os.path.splitext(
                    args.csv_fd.name
                )

            workbook_path = \
                root + '.xlsx'

        if not args.force:
            if os.path.exists(workbook_path):
                while True:
                    response = \
                        input(
                            """Output file '{0}' exists.\
  Continue? (y/n) """.format(
                                workbook_path
                            )
                        )

                    if response == 'y':
                        break
                    elif response == 'n':
                        sys.exit()

        csv2xlsx.transform(
            args.csv_fd,
            workbook_path,
            workbook_settings=workbook_settings
        )

        args.csv_fd.close()



#!/usr/bin/env python
#   -*- coding: utf-8 -*-

from setuptools import setup
from setuptools.command.install import install as _install

class install(_install):
    def pre_install_script(self):
        pass

    def post_install_script(self):
        pass

    def run(self):
        self.pre_install_script()

        _install.run(self)

        self.post_install_script()

if __name__ == '__main__':
    setup(
        name = 'dhk.csv2xlsx',
        version = '2.0.0',
        description = 'Read a CSV file and write an XLSX file with optional formatting.',
        long_description = 'dhk.csv2xlsx\n============\n\n[![GitHub](https://img.shields.io/badge/github-python--csv2xlsx-blue?logo=github)](https://github.com/DavidKiesel/python-csv2xlsx)\n\n[![Latest Version](https://img.shields.io/pypi/v/dhk.csv2xlsx?logo=pypi)](https://pypi.org/project/dhk.csv2xlsx/)\n[![Python Versions](https://img.shields.io/pypi/pyversions/dhk.csv2xlsx?logo=pypi)](https://pypi.org/project/dhk.csv2xlsx/)\n\n[![Downloads Per Day](https://img.shields.io/pypi/dd/dhk.csv2xlsx?logo=pypi)](https://pypi.org/project/dhk.csv2xlsx/)\n[![Downloads Per Week](https://img.shields.io/pypi/dw/dhk.csv2xlsx?logo=pypi)](https://pypi.org/project/dhk.csv2xlsx/)\n[![Downloads Per Month](https://img.shields.io/pypi/dm/dhk.csv2xlsx?logo=pypi)](https://pypi.org/project/dhk.csv2xlsx/)\n\n# Introduction\n\n`dhk.csv2xlsx` is a Python command-line tool for reading a CSV file and writing an XLSX file with optional formatting.\nIt leverages the Python standard library [`csv`](https://docs.python.org/3/library/csv.html) module and the [`XlsxWriter`](https://pypi.org/project/XlsxWriter/) package.\n\n# Simple Installation\n\nA pedestrian command for installing the package is given below.\nAlternatively, for a more rewarding installation exercise, see section [Recommended Installation](#recommended-installation).\n\n```bash\npip install dhk.csv2xlsx\n```\n\n# Usage\n\n```console\n$ csv2xlsx --help\nusage: csv2xlsx [-h] [--force] [--output OUTPUT_FILE]\n                [--settings-file SETTINGS_FILE] [--verbose]\n                [--generate-settings-file | --transform-csv]\n                [CSV_FILE]\n\nRead a CSV file and write an XLSX file.\n\nThe program exposes much of the formatting functionality of the XlsxWriter\npackages Workbook class through the optional SETTINGS_FILE.  For details about\nthe Workbook class, see https://xlsxwriter.readthedocs.io/workbook.html.  For\ndetails about the Worksheet class, see\nhttps://xlsxwriter.readthedocs.io/worksheet.html.  For details about the Format\nclass, see https://xlsxwriter.readthedocs.io/format.html.\n\npositional arguments:\n  CSV_FILE              CSV file\n\noptions:\n  -h, --help            show this help message and exit\n  --force, -f           force; suppress prompts\n  --output, -o OUTPUT_FILE\n                        output file; default: CSV_FILE - .csv + .xlsx\n  --settings-file, -s SETTINGS_FILE\n                        settings file\n  --verbose, -v         verbose\n  --generate-settings-file, -g\n                        generate settings file; file defaults to\n                        sample.settings.json\n  --transform-csv, -t   transform CSV to XLSX; default True\n\nexamples:\n\n    csv2xlsx \\\n        --generate-settings-file\n\n    csv2xlsx \\\n        CSV_FILE\n\n    csv2xlsx \\\n        --settings-file SETTINGS_FILE \\\n        CSV_FILE\n\n    csv2xlsx \\\n        --settings-file SETTINGS_FILE \\\n        --output OUTPUT \\\n        CSV_FILE\n\nexecution details:\n\nThe Workbook class instance attribute `options` is set to `{\'constant_memory\':\nTrue}` merged with the SETTINGS_FILE `workbook_options` dictionary.\n\nThe Workbook class instance attribute `formats[0]` stores a default Format\nobject.  The SETTINGS_FILE `workbook_format` dictionary can be used to set the\nattributes of this default Format object.  The available dictionary keywords\ncorrespond to the list of properties in the table at\nhttps://xlsxwriter.readthedocs.io/format.html#format-methods-and-format-properties.\n\nThe Workbook class instance method `add_format()` is used to add Format class\ninstances to the Workbook class instance.  The program iterates over each of\nthe SETTINGS_FILE `cell_format_settings` dictionary key-value pairs.  The\nSETTINGS_FILE `workbook_format` dictionary is merged with the value of the\n`cell_format_settings` key-value pair, and this merged value is the argument to\n`add_format()`.  The key-value pair formed by the key from the\n`cell_format_settings` key-value pair and the `add_format()` return value is\nstored in a `cell_formats` dictionary for later use when setting Worksheet\nclass instance columns and rows.\n\nThe Worksheet class instance method `set_column()` is used to set the width,\ncell format, and options for specified columns.  The program iterates over each\nof the SETTINGS_FILE `column_settings` list elements.  For each element, the\n`first_col` and `last_col` arguments to `set_column()` can be specified in\nabsolute terms via a `columns` key-value pair with the string value specified\nby ordinal number (`\'1\'`, `\'1:2\'`, etc.) or by `A1` style notation (`\'A\'`,\n`\'A:B\'`).  Alternatively, if the `columns` key-value pair is not present, then\n`first_col` is simply the next column, and `last_col` is either the same as\n`first_col` or is set based on a `span` key-value pair.  The `width` argument\nto `set_column()` is set to the value of the `width` key-value pair, defaulting\nto 10 if not specified.  The `cell_format` argument to `set_column()` is set\nbased on the value of the `cell_format` key-value pair.  This value must\ncorrespond to the key of one of the SETTINGS_FILE `cell_format_settings`\nkey-value pairs.  If a `cell_format` key-value pair is not provided, then the\nargument defaults to the Workbook class instance attribute `formats[0]`.  The\n`options` argument to `set_column()` is set to the value of the `options`\nkey-pair.  The value of the `data_type` key-pair is stored for each column for\nlater use when parsing CSV data.\n\nThe Worksheet class instance method `set_row()` is used to set the cell format\nfor specified rows.  The program iterates over each of the SETTINGS_FILE\n`row_settings` list elements.  For each element, `first_row` and `last_row` can\nbe specified in absolute terms via a `rows` key-value pair with the string\nvalue specified by ordinal number (`\'1\'`, `\'1:2\'`, etc.).  Alternatively, if\nthe `rows` key-value pair is not present, then `first_row` is simply the next\ncolumn, and `last_row` is either the same as `first_row` or is set based on a\n`span` key-value pair.  For each `row` in the sequence determined by\n`first_row` and `last_row`, `set_row()` is called with `row` as the first\npositional argument and the argument `cell_format` set based on the value of\nthe `cell_format` key-value pair.  This value must correspond to the key of one\nof the SETTINGS_FILE `cell_format_settings` key-value pairs.  If a\n`cell_format` key-value pair is not provided, then the argument defaults to\nthe Workbook class instance attribute `formats[0]`.  The value of the\n`data_type` key-pair is stored for each row for later use when parsing CSV\ndata.\n\nEach row of the CSV file is read.  The value of each column of the row is\nhandled.  If there was a `data_type` specified in the SETTINGS_FILE for the\nrow, then that is used.  Otherwise, an attempt is made to get a `data_type`\nthat was specified in the SETTINGS_FILE for the column.  Valid values for\n`data_type` are: `decimal`, `bool`, `date`, `datetime`, `formula`, and\n`string`.  If `data_type` was not specified or an exception occurs when\nattempting to parse the CSV value in terms of the given `data_type`, then the\nCSV value is written as a `string`.\n\nThe Worksheet class instance method `autofilter` is called with arguments based\non the `autofilter` object in the SETTINGS_FILE `workbook_settings` dictionary.\nBoth row-column notation (`(first_row, first_col, last_row, last_col)`) and\n`A1` style notation (`\'A1:D11\'`) are supported.  If row-column notation is\nused, then a negative number is replaced with the maximum row or column\navailable in the data.\n\nThe Worksheet class instance method `freeze_panes` is called with arguments\nbased on the `freeze_panes` object in the SETTINGS_FILE `workbook_settings`\ndictionary.  Both row-column notation (`(row, col[, top_row, left_col])`) and\n`A1` style notation (`\'A2\'`) are supported.\n```\n\n# Recommended Installation\n\n[`pyenv`](https://github.com/pyenv/pyenv) is a tool for installing multiple Python environments and controlling which one is in effect in the current shell.\n\n[`pipx`](https://github.com/pipxproject/pipx) is a tool for installing and running Python applications in isolated environments.\n\nAssuming these have been installed correctly...\n\n## Install Python Under `pyenv`\n\nThe version of Python under which this package was last developed and tested is stored in [`.python-version`](https://raw.githubusercontent.com/DavidKiesel/python-csv2xlsx/refs/heads/main/.python-version).\n\nTo capture this Python version to a shell variable, execute the commands below.\n`PYTHON_VERSION` should be set to something like `3.13.3`.\n\n```bash\nPYTHON_VERSION="$(\n    wget \\\n        -O - \\\n        https://raw.githubusercontent.com/DavidKiesel/python-csv2xlsx/refs/heads/main/.python-version\n)"\n\necho "$PYTHON_VERSION"\n```\n\nTo determine if the `.python-version` version of Python has already been installed under `pyenv`, execute the command below.\nIf it has not been installed, then a warning message will be displayed.\n\n```bash\nPYENV_VERSION="$PYTHON_VERSION" \\\npython --version\n```\n\nIf it has already been installed, then proceed to section [Install Package Using `pipx`](#install-package-using-pipx).\n\nOtherwise, to install the given version of Python under `pyenv`, execute the command below.\n\n```bash\npyenv install "$PYTHON_VERSION"\n```\n\nIf the install was successful, then proceed to section [Install Package Using `pipx`](#install-package-using-pipx).\n\nIf instead there is a warning that the definition was not found, then you will need to upgrade `pyenv`.\n\nIf `pyenv` was installed through a package manager, then consider upgrading it through that package manager.\nFor example, if `pyenv` was installed through `brew`, then execute the commands below.\n\n```bash\nbrew update\n\nbrew upgrade pyenv\n```\n\nAlternatively, you could attempt to upgrade `pyenv` through the command below.\n\n```bash\npyenv update\n```\n\nOnce `pyenv` has been upgraded, to install the given version of Python under `pyenv`, execute the command below.\n\n```bash\npyenv install "$PYTHON_VERSION"\n```\n\n## Install Package Using `pipx`\n\nOnly proceed from here if the instructions in section [Install Python Under `pyenv`](#install-python-under-pyenv) have been completed successfully.\n\nAt this point, shell variable `PYTHON_VERSION` should already contain the appropriate Python version.\nIf not, execute the commands below.\n\n```bash\nPYTHON_VERSION="$(\n    wget \\\n        -O - \\\n        https://raw.githubusercontent.com/DavidKiesel/python-csv2xlsx/refs/heads/main/.python-version\n)"\n\necho "$PYTHON_VERSION"\n```\n\nTo install the package hosted at PyPI using `pipx`, execute the command below.\n\n```bash\npipx \\\n    install \\\n    --python "$(PYENV_VERSION="$PYTHON_VERSION" pyenv which python3)" \\\n    dhk.csv2xlsx\n```\n',
        long_description_content_type = 'text/markdown',
        classifiers = [
            'Development Status :: 3 - Alpha',
            'Environment :: Console',
            'Intended Audience :: Developers',
            'Programming Language :: Python',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.9',
            'Programming Language :: Python :: 3.10',
            'Programming Language :: Python :: 3.11',
            'Programming Language :: Python :: 3.12',
            'Programming Language :: Python :: 3.13'
        ],
        keywords = '',

        author = 'David Harris Kiesel',
        author_email = 'david.sw@suddenthought.net',
        maintainer = 'David Harris Kiesel',
        maintainer_email = 'david.sw@suddenthought.net',

        license = 'MIT',

        url = 'https://github.com/DavidKiesel/python-csv2xlsx',
        project_urls = {
            'Homepage': 'https://github.com/DavidKiesel/python-csv2xlsx'
        },

        scripts = ['scripts/csv2xlsx'],
        packages = ['dhk.csv2xlsx'],
        namespace_packages = [],
        py_modules = [],
        entry_points = {},
        data_files = [],
        package_data = {},
        install_requires = ['XlsxWriter==3.2.3'],
        dependency_links = [],
        zip_safe = True,
        cmdclass = {'install': install},
        python_requires = '>=3.9',
        obsoletes = [],
    )
